"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GeoCoder = void 0;
const leaflet_1 = __importDefault(require("leaflet"));
const Control_1 = __importDefault(require("./Control"));
const PopupContent_1 = __importDefault(require("./PopupContent"));
class Utils {
    /**
     * 引数で指定した位置にピンをポップアップマーカーを立てる
     */
    static createPin = (map, latlng, data) => {
        const marker = leaflet_1.default.marker(latlng, {
            alt: `${data.pref} ${data.city} ${data.town}`
        });
        const popup = leaflet_1.default.popup().setContent(new PopupContent_1.default({
            "都道府県": data.pref,
            "市区町村": data.city,
            "町名": data.town,
            "緯度": latlng.lat,
            "経度": latlng.lng
        }));
        marker.bindPopup(popup); // popupをmarkerに関連付ける
        marker.setLatLng(latlng); // markerに緯度経度を設定
        marker.addTo(map); // markerをmapに追加
        marker.openPopup(); // popupを開く 
    };
    /**
     * 引数で指定した位置に地図の中心を移動する
     */
    static moveTo = (map, location) => {
        map.flyTo(location);
    };
}
;
class GeoCoder extends leaflet_1.default.Control {
    _div;
    normalizer;
    map;
    constructor(normalizer, options) {
        super(options);
        this.normalizer = normalizer;
    }
    onAdd = (map) => {
        this.map = map;
        this._div = leaflet_1.default.DomUtil.create("div", "leaflet-community-geocoder");
        this._div.appendChild(new Control_1.default(this.search));
        return this._div;
    };
    /**
     * Leaflet地図とGeoCoderを関連付けます
     */
    on = (map) => {
        this.map = map;
    };
    /**
     * 引数に指定した住所周辺の地図を表示します
     * @param address 住所
     */
    search = async (address) => {
        if (this.map == undefined) {
            throw Error("searchメソッドを呼ぶ前に、GeoCoderをL.Mapオブジェクトと関連付ける必要があります");
        }
        else {
            try {
                const result = await this.normalizeAddress(address);
                Utils.moveTo(this.map, result.latlng);
                Utils.createPin(this.map, result.latlng, result.parsedAddress);
            }
            catch (error) {
                alert(error);
            }
        }
    };
    /**
     * コンストラクタで指定した住所パーサーと、@geolonia/japanese-addressesのデータを利用してジオコーディングを行ないます
     */
    normalizeAddress = async (address) => {
        const result = await this.normalizer(address);
        if (result.level >= 3) {
            // level 3 まで判別できた場合には、緯度経度情報も返ってくる
            // https://github.com/geolonia/normalize-japanese-addresses/pull/113
            return {
                latlng: leaflet_1.default.latLng({ lat: result.lat, lng: result.lng }),
                parsedAddress: { pref: result.pref, city: result.city, town: result.town }
            };
        }
        else if (result.level == 2) {
            // level 2 までしか判別できなかった場合は、
            // 別途データベースに問い合わせ、大まかな位置情報を取得する
            const url = `https://geolonia.github.io/japanese-addresses/api/ja/${result.pref}/${result.city}.json`;
            const json = await fetch(url).then(response => response.json());
            const destination = json[0];
            return {
                latlng: leaflet_1.default.latLng({ lat: parseFloat(destination.lat), lng: parseFloat(destination.lng) }),
                parsedAddress: { pref: result.pref, city: result.city, town: result.town }
            };
        }
        else {
            throw Error("もう少し詳しい住所を入力してください");
        }
    };
}
exports.GeoCoder = GeoCoder;
//# sourceMappingURL=GeoCoder.js.map